"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SingleTransform = void 0;
/**
 * Abstract class to simplify implementation of a VFS that generates a single
 * file from one or more inputs.
 */
class SingleTransform {
    constructor(options) {
        var _a, _b;
        this.target = options.target;
        this.inputs = options.inputs;
        this.isScript = (_a = options.isScript) !== null && _a !== void 0 ? _a : false;
        this.isModule = (_b = options.isModule) !== null && _b !== void 0 ? _b : false;
    }
    read(target) {
        if (target == this.target) {
            return this.generateSingle();
        }
        else {
            return null;
        }
    }
    info(path) {
        if (path != this.target) {
            return null;
        }
        const version = this.getVersion();
        if (version == null) {
            return null;
        }
        return {
            path,
            version,
            generated: true,
            isScript: this.isScript,
            isModule: this.isModule
        };
    }
    listFiles() {
        return [this.target];
    }
    getTargetsFor(source) {
        if (this.inputs.includes(source)) {
            return [this.target];
        }
        return null;
    }
}
exports.SingleTransform = SingleTransform;
//# sourceMappingURL=SingleTransform.js.map