"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.cachedSingleGenerate = exports.cachedGenerate = exports.getInputsVersions = void 0;
function getInputsVersions(fs, inputs) {
    let versions = [];
    for (let input of inputs) {
        const version = fs.getVersion(input);
        if (version == null) {
            return null;
        }
        versions.push(`${input}:${version}`);
    }
    return versions.join(':');
}
exports.getInputsVersions = getInputsVersions;
function cachedGenerate(options) {
    const cache = {};
    return (target) => {
        var _a;
        const inputs = options.getInputsFor(target);
        if (inputs == null) {
            return null;
        }
        const version = getInputsVersions(options.fs, inputs);
        if (version == null) {
            return null;
        }
        if (((_a = cache[target]) === null || _a === void 0 ? void 0 : _a.version) == version) {
            return cache[target].data;
        }
        const generated = options.generate(target);
        cache[target] = { version, data: generated };
        return generated;
    };
}
exports.cachedGenerate = cachedGenerate;
function cachedSingleGenerate(options) {
    let cache = null;
    return () => {
        const version = options.version();
        if (version == null) {
            return null;
        }
        if ((cache === null || cache === void 0 ? void 0 : cache.version) == version) {
            return cache.data;
        }
        const generated = options.generate();
        cache = { version, data: generated };
        return generated;
    };
}
exports.cachedSingleGenerate = cachedSingleGenerate;
//# sourceMappingURL=caching.js.map