"use strict";
// !!! Code duplication warning !!!
// This currently duplicates work in @journeyapps/cloudcode-build.
// Currently the implementation here requires SourceMaps, which will require more tweaks over time.
// Once this implementation has stabilized, it should be moved to @journeyapps/parser-schema, then the same implementation
// used for this and cloudcode-build.
Object.defineProperty(exports, "__esModule", { value: true });
exports.safeProperty = exports.isValidProperty = exports.modelInterfaceName = exports.typeRef = exports.schemaToDefinitions = exports.mapType = void 0;
const reservedKeywords_1 = require("./reservedKeywords");
const source_utils_1 = require("@journeyapps/source-utils");
const TYPE_MAP = {
    text: 'string',
    integer: 'number',
    'single-choice-integer': 'number',
    'single-choice': 'string',
    boolean: 'boolean',
    number: 'number',
    date: 'Day',
    datetime: 'Date',
    'multiple-choice-integer': 'number[]',
    'multiple-choice': 'string[]',
    location: 'Location',
    attachment: 'Attachment',
    track: 'Attachment'
};
function isArrayType(type) {
    return type.name == 'array';
}
function isQueryType(type) {
    return type.name == 'query';
}
/**
 * Given a Journey type, return a TypeScript type name.
 *
 * If the type is unknown, we return 'any'.
 */
function mapType(type) {
    if (type == null) {
        // Safety fallback.
        return 'any';
    }
    else if (type.name in TYPE_MAP) {
        return TYPE_MAP[type.name];
    }
    else if (type.isObject) {
        return typeRef(type.name);
    }
    else if (isArrayType(type) && type.objectType) {
        return `${typeRef(type.objectType.name)}[]`;
    }
    else if (isQueryType(type) && type.objectType) {
        return `Query<${typeRef(type.objectType.name)}>`;
    }
    else {
        return 'any';
    }
}
exports.mapType = mapType;
function indentation(levels) {
    let r = '';
    for (let i = 0; i < levels; i++) {
        r += '  ';
    }
    return r;
}
function trimEnd(line) {
    let i = line.length;
    while (i > 0) {
        if (line.charAt(i - 1) == ' ') {
            i -= 1;
        }
        else {
            break;
        }
    }
    return line.substring(0, i);
}
function indent(text, levels = 1) {
    if (Array.isArray(text)) {
        return indent(text.join('\n'), levels);
    }
    const lines = text.split('\n');
    return lines.map((line) => trimEnd(indentation(levels) + line)).join('\n');
}
function sourceNode(text, file, sourceElement) {
    if ((sourceElement === null || sourceElement === void 0 ? void 0 : sourceElement.openStart) && (sourceElement === null || sourceElement === void 0 ? void 0 : sourceElement.openEnd)) {
        const start = sourceElement === null || sourceElement === void 0 ? void 0 : sourceElement.openStart;
        const length = (sourceElement === null || sourceElement === void 0 ? void 0 : sourceElement.openEnd) - start;
        return new source_utils_1.SourceNode({ content: text, sourceFile: file, sourceStart: start, sourceLength: length });
    }
    else {
        return new source_utils_1.SourceNode({ content: text, sourceFile: file });
    }
}
function schemaToDefinitions(schema, options) {
    var _a;
    const models = [];
    const modelNames = Object.keys(schema.objects);
    const file = (_a = options.sourcePath) !== null && _a !== void 0 ? _a : 'file:///schema.xml';
    const { dbs, mobileUserType } = options;
    function modelRef(name, text) {
        const type = schema.objects[name];
        return sourceNode(text, file, type.sourceElement);
    }
    function fieldRef(field, text) {
        return sourceNode(text, file, field.sourceElement);
    }
    function promise(type) {
        if (!options.asynchronous) {
            return type;
        }
        else {
            return `Promise<${type}>`;
        }
    }
    for (let modelName in schema.objects) {
        const model = schema.objects[modelName];
        let attrs = [];
        for (let attributeName of Object.keys(model.attributes)) {
            let attribute = model.attributes[attributeName];
            attrs.push(fieldRef(attribute, `/** ${attribute.label} (${attribute.sourceTypeName}) */\n${safeProperty(attributeName)}: ${mapType(attribute.type)};`));
            if (attribute.type && attribute.type.name == 'attachment') {
                attrs.push(fieldRef(attribute, `/** ID of ${attribute.label}. */\nreadonly ${attributeName}_id: string;`));
            }
        }
        for (let relName in model.belongsTo) {
            let rel = model.belongsTo[relName];
            if (!rel.foreignType) {
                continue;
            }
            attrs.push(fieldRef(model.belongsToVars[relName], `/** Lookup related ${rel.foreignType.label}. */
        ${safeProperty(relName)}(): ${promise(typeRef(rel.foreignType.name))};
        /** Set related ${rel.foreignType.label}. */
        ${safeProperty(relName)}(value: ${typeRef(rel.foreignType.name)}): void;
        /** ID of related ${rel.foreignType.label}. */
        ${safeProperty(relName + '_id')}: string;`));
        }
        for (let relName in model.hasMany) {
            let rel = model.hasMany[relName];
            if (!rel.objectType) {
                continue;
            }
            attrs.push(fieldRef(model.hasManyVars[relName], `/** Query for related ${rel.objectType.label}. */
        readonly ${safeProperty(relName)}: Query<${typeRef(rel.objectType.name)}>;`));
        }
        const modelTs = (0, source_utils_1.sourceMappedTemplate)('schema.xml') `
interface ${modelRef(modelName, modelInterfaceName(modelName))} extends DatabaseObject {
${attrs}
}
`;
        models.push(modelTs);
    }
    return (0, source_utils_1.sourceMappedTemplate)('schema.xml') `
// Automatically generated from the data-model.
// DO NOT MODIFY.
// Any modifications will be lost when building.

import { Database, Query, Collection, DatabaseObject, Attachment, Location, Day } from '@journeyapps/db';

interface DB extends Database {
${modelNames.map((name) => modelRef(name, indent(`${safeProperty(name)}: Collection<${typeRef(name)}>;`)))}
}


declare global {

  namespace DB {
${models}
  }

${dbs.map((name) => `  const ${name}: DB\n`).join('')}

${mobileUserType ? `  const user: DB.${mobileUserType};` : ''}
}`;
}
exports.schemaToDefinitions = schemaToDefinitions;
/**
 * Get a reference to a model type/interface.
 *
 * "user" => "DB.user".
 * "class" => DB._class".
 *
 * @param modelName Name of the model.
 */
function typeRef(modelName) {
    return `DB.${modelInterfaceName(modelName)}`;
}
exports.typeRef = typeRef;
/**
 * Given a model name, return a safe identifier for the model interface.
 *
 * Generally the interface name should equal the model name. However, in some cases the model name is a reserved
 * keyword, in which case we prepend an underscore.
 *
 * "user" => "user"
 * "class" => "_class".
 *
 * @param name Name of the model.
 */
function modelInterfaceName(name) {
    // If the name is a reserved word, we prepend an underscore to the type name.
    // This only applies to the type, not the model.
    // Effectively we'd get `var c: DB._class = DB.class.create()`
    if ((0, reservedKeywords_1.isReservedTypeName)(name)) {
        return `_${name}`;
    }
    return name;
}
exports.modelInterfaceName = modelInterfaceName;
const PROPERTY_REGEXP = /^(?![0-9])[a-zA-Z0-9$_]+$/;
/**
 * Returns true if an identifier is valid as a property name in JavaScript / TypeScript.
 * In other words, it can be directly used as obj.prop_name_here, versus the workaround of obj["prop name here"].
 *
 * General restrictions are that it cannot start with a number, and cannot contain spaces or special characters.
 * @param name - The identifier
 */
function isValidProperty(name) {
    return PROPERTY_REGEXP.test(name);
}
exports.isValidProperty = isValidProperty;
/**
 * Return a property name safe to embed in a TS definition file.
 *
 * Where feasible, we return the property directly. If it's not a valid name directly, we wrap it in quotes.
 *
 * my_prop123 => my_prop123
 * 1time => "1time"
 * multiple words => "multiple words"
 *
 * @param name - The property name.
 */
function safeProperty(name) {
    // We may get invalid property names. In that case the datamodel should have validation errors, but we still
    // attempt to handle them gracefully here, by wrapping them in quotes.
    if (isValidProperty(name)) {
        return name;
    }
    else {
        return JSON.stringify(name);
    }
}
exports.safeProperty = safeProperty;
//# sourceMappingURL=schemaToDefinitions.js.map