import { ListOptions, SourceNode, SyncFileSystem } from '@journeyapps/source-utils';
import { VFS } from './VFS';
import { SourceEntry } from './SourceEntry';

/**
 * Serve source files directly.
 *
 * Wrap the fs with PrefixedFileSystem to serve under a prefix.
 */
export class SourceVFS implements VFS {
  /**
   *
   * @param fs - the source FileSystem
   * @param scriptDir - list files in this dir as scripts
   * @param scriptFilter - filter for scripts
   */
  constructor(private fs: SyncFileSystem, private scriptDir: string, private scriptFilter: ListOptions) {}

  getTargetsFor(source: string): string[] | null {
    return null;
  }

  info(path: string): SourceEntry | null {
    if (this.fs.existsSync(path)) {
      const version = this.fs.getVersion(path);
      if (version == null) {
        return null;
      }
      return {
        path: path,
        generated: false,
        isScript: true,
        isModule: true,
        version: version
      };
    } else {
      return null;
    }
  }

  listFiles(): string[] {
    return this.fs.listFilesRecursivelySync(this.scriptDir, this.scriptFilter).sort();
  }

  read(path: string): SourceNode | null {
    if (this.fs.existsSync(path)) {
      return SourceNode.createDirectNode({ content: this.fs.readTextSync(path), sourceFile: path });
    } else {
      return null;
    }
  }
}
