import * as fs from 'fs';
import * as path from 'path';
import * as glob from 'glob';

export function loadPackage(name: string, prefix?: string, context?: { filter?: RegExp }, packageData?: any) {
  let result: Record<string, string> = {};
  const packagePath = require.resolve(name + '/package.json');
  const filter = context?.filter;

  const dir = path.join(packagePath, '..');
  let pattern = '**/*.d.ts';
  if (prefix) {
    pattern = `${prefix}/${pattern}`;
  }
  const files = glob.sync(pattern, { cwd: dir });
  for (let file of files) {
    if (filter == null || filter.test(file)) {
      result[`${name}/${file}`] = fs.readFileSync(path.join(dir, file), 'utf-8');
    }
  }

  const packageData2 = JSON.parse(fs.readFileSync(packagePath, 'utf-8'));
  const typings = packageData2.appTypings ?? packageData2.typings ?? packageData2.types;
  result[`${name}/package.json`] = JSON.stringify({ name, typings });
  return result;
}

export function loadFile(path: string, context?: any) {
  return fs.readFileSync(require.resolve(path), 'utf-8');
}

export interface RequireContext {
  (name: string): any;
  context(path: string, includeSubfolders?: boolean, filter?: RegExp): any;
}

export const requireContext: RequireContext = function () {
  return null;
};

requireContext.context = (path: string, includeSubfolders?: boolean, filter?: RegExp) => {
  return {
    path,
    includeSubfolders,
    filter
  };
};
